import { readdir } from 'node:fs/promises';
import { openPostMarkdownFile, parsePostMetadata } from '@blog/post/post-reader';
import { PostMeta } from '@blog/model/PostMeta';
import db from '@blog/db/memory';

const createPostSql: string = `
CREATE TABLE posts (
  id INTEGER PRIMARY KEY,
  title TEXT NOT NULL,
  slug TEXT NOT NULL,
  is_draft INTEGER DEFAULT 0,
  description TEXT,
  published_date DATE,
  raw_content TEXT
)
`;

const createAuthorSql: string = `
CREATE TABLE authors (
  id INTEGER PRIMARY KEY,
  name TEXT,
  email TEXT
)
`;

const createTagsSql: string = `
CREATE TABLE tags (
  id INTEGER PRIMARY KEY,
  tag TEXT UNIQUE
)
`;

const createPostsTagsSql: string = `
CREATE TABLE posts_tags (
  tag_id TEXT,
  post_id INTEGER,

  FOREIGN KEY (tag_id) REFERENCES tags(id) ON DELETE CASCADE,
  FOREIGN KEY (post_id) REFERENCES posts(id) ON DELETE CASCADE 
)
`;

const createPostHtmlCache = `
CREATE TABLE post_html_cache (
  id INTEGER PRIMARY KEY,
  post_id INTEGER,
  content TEXT,

  FOREIGN KEY (post_id) REFERENCES posts(id) ON DELETE CASCADE
);
`;

export async function setupDb(): Promise<void> {
  console.log({ message: "Setting up the Database..." });
  console.log({ message: "Creating the Posts table..." });
  db.query(createPostSql).run();

  console.log({ message: "Creating Authors table..." });
  db.query(createAuthorSql).run();

  console.log({ message: "Creating Tags table..." });
  db.query(createTagsSql).run();

  console.log({ message: "Creating Post and Tags relationships..." });
  db.query(createPostsTagsSql).run();

  console.log({ message: "Creating HTML cache table..." });
  db.query(createPostHtmlCache).run();
}


export async function readPostsToDatabase(postPath: string): Promise<void> {
  const posts = new Array();
  const createPostSql: string = `
INSERT INTO posts (title, slug, description, is_draft, published_date, raw_content)
VALUES ($title, $slug, $description, $is_draft, $published_date, $raw_content);
`;

  const fileList = await readdir(postPath);
  for (const file of fileList) {
    const contents: string = await openPostMarkdownFile(postPath + "/" + file);
    const post = parsePostMetadata(contents);
  
    const postQuery = db.prepare(createPostSql);
    postQuery.run({
      $title: post.meta.title,
      $slug: file,
      $description: post.meta.description,
      $is_draft: post.meta.draft ?? false,
      $published_date: post.meta.date.toString(),
      $raw_content: post.content
    });
    console.log({ message: "Added " + post.meta.title + " to the database." });
  }
}
